@@ -5,7 +5,7 @@
 # SPDX-License-Identifier: BSD-3-Clause
 
 from collections import defaultdict
-from typing import Iterable, Tuple, List, Optional
+from typing import Iterable, Tuple, List, Optional, Union
 
 from docutils import nodes
 from docutils.statemachine import StringList
@@ -154,7 +154,7 @@ class VHDLEntityIOGenericDirective(SphinxDirective):
     title: str
     id_title: str
 
-    def get_fields_from_definition(self, definition: str) -> Tuple[str, str, str]:
+    def get_fields_from_definition(self, definition: str) -> Union[Tuple[str, str, str], Tuple[str, str, str, str]]:
         raise NotImplementedError
 
     def run(self):
@@ -164,43 +164,96 @@ class VHDLEntityIOGenericDirective(SphinxDirective):
 
         head = nodes.thead()
         body = nodes.tbody()
-        group += nodes.colspec(colwidth=10)
-        group += nodes.colspec(colwidth=10)
-        group += nodes.colspec(colwidth=10)
-        group += nodes.colspec(colwidth=70)
-        group += head
-        group += body
+
+        index = 0
+        has_groups = False
+        has_group_desc = False
+
+        # Get all fields from ports or generics
+        # Then check if ports or generics have groups and group description
+        while len(self.content) > index:
+            if len(self.content[index]) > 0 and not self.content[index][0].isspace():
+                fields = self.get_fields_from_definition(self.content[index])
+            index += 1
+        index = 0
 
         row: Optional[nodes.row]
         row = nodes.row()
-        row += nodes.entry('', nodes.paragraph('', nodes.Text(self.table_headers[0])))
+        # Define first line of table
         row += nodes.entry('', nodes.paragraph('', nodes.Text(self.table_headers[1])))
         row += nodes.entry('', nodes.paragraph('', nodes.Text(self.table_headers[2])))
         row += nodes.entry('', nodes.paragraph('', nodes.Text(self.table_headers[3])))
+        row += nodes.entry('', nodes.paragraph('', nodes.Text(self.table_headers[4])))
         head += row
+
         row = None
         current_description_lines = StringList()
-        index = 0
         description_entry = nodes.entry('')
+
+        # Define table fields and their dimensions
+        group += nodes.colspec(colwidth=10)
+        group += nodes.colspec(colwidth=10)
+        group += nodes.colspec(colwidth=10)
+        group += nodes.colspec(colwidth=100)
+
+        # Insert group header to table
+        group += head
+        group += body
+        current_group = ''
+
+        # Fill the table with content
         while len(self.content) > index:
             if len(self.content[index]) > 0 and not self.content[index][0].isspace():
+
                 if row is not None:
                     body += row
                 row = nodes.row()
                 fields = self.get_fields_from_definition(self.content[index])
-                row_id = f'vhdl-{self.id_title}-{self.arguments[0].lower()}-{fields[0].lower()}'
-                self.env.domaindata['vhdl']['refs'][self.id_title][fields[0].lower()].append(
+                has_groups = has_groups or len(fields) >= 4
+
+                # If there is a group then fill first line of table with name of group, separators and description
+                if has_groups:
+                    if current_group != (fields[0]):
+                        current_group = (fields[0])
+                        has_group_desc = len(autodoc.groups_desc[current_group]) != 0 and has_groups
+
+                        # Create nodes that contains name and description of group
+                        group_name = nodes.entry('')
+                        group_desc = nodes.entry('')
+                        self.state.nested_parse(StringList(initlist=[fields[0].split(' ', 1)[1]]), 0, group_name)
+                        self.state.nested_parse(StringList(autodoc.groups_desc[current_group]), 0, group_desc)
+
+                        # Create row that contains information about group (name, description and separators)
+                        separator = "====="
+                        par = [
+                            nodes.entry('', nodes.paragraph('', nodes.Text(separator))),
+                            group_name,
+                            nodes.entry('', nodes.paragraph('', nodes.Text(separator))),
+                            group_desc if has_group_desc else nodes.entry('', nodes.paragraph('', nodes.Text(separator))),
+                        ]
+                        for p in par:
+                            row += p
+                        body += row
+                        pass
+                    row = nodes.row()
+
+                # Fill the table with content
+                if len(fields) == 3:
+                    fields = "", fields[0], fields[1], fields[2]
+                row_id = f'vhdl-{self.id_title}-{self.arguments[0].lower()}-{fields[1].lower()}'
+                self.env.domaindata['vhdl']['refs'][self.id_title][fields[1].lower()].append(
                     (self.arguments[0].lower(), (self.env.docname, row_id)))
                 row['ids'].append(row_id)
-                row += nodes.entry('', nodes.paragraph('', nodes.Text(fields[0])))
 
-                refnode = pending_xref(refdomain='vhdl', reftype='type', reftarget=fields[1])
-                refnode += nodes.Text(fields[1])
+                row += nodes.entry('', nodes.paragraph('', nodes.Text(fields[1])))
+
+                refnode = pending_xref(refdomain='vhdl', reftype='type', reftarget=fields[2])
+                refnode += nodes.Text(fields[2])
                 type_node = nodes.entry('')
-                self.state.nested_parse(StringList(initlist=[fields[1]]), 0, type_node)
+                self.state.nested_parse(StringList(initlist=[fields[2]]), 0, type_node)
                 row += type_node
 
-                row += nodes.entry('', nodes.paragraph('', nodes.Text(fields[2])))
+                row += nodes.entry('', nodes.paragraph('', nodes.Text(fields[3])))
 
                 self.state.nested_parse(current_description_lines.get_indented()[0], 0, description_entry)
                 description_entry = nodes.entry('')
@@ -212,32 +265,58 @@ class VHDLEntityIOGenericDirective(SphinxDirective):
         if row is not None:
             self.state.nested_parse(current_description_lines.get_indented()[0], 0, description_entry)
             body += row
-
         return [addnodes.desc_name(text=self.title), table]
 
 
 class VHDLPortsDirective(VHDLEntityIOGenericDirective):
     id_title = 'portsignal'
     title = 'Ports'
-    table_headers = 'Port', 'Type', 'Mode', 'Description'
+    table_headers = 'Group', 'Port', 'Type', 'Mode', 'Description'
+
+    def get_fields_from_definition(self, definition: str) -> Union[Tuple[str, str, str], Tuple[str, str, str, str]]:
+        try:
+            if definition.strip().startswith("SPHINXGRP "):
+                return definition.split(maxsplit=1)[1].strip(), "", "", ""
+            elif '}' not in definition:
+                return (
+                    definition.split(":")[0].strip(),
+                    definition.split(":", 1)[1].strip().split(maxsplit=1)[1],
+                    definition.split(":", 1)[1].strip().split()[0]
+            )
+            else:
+                return(
+                    definition.split("}")[0].strip(),
+                    definition.split("}")[1].split(":")[0].strip(),
+                    definition.split("}")[1].split(":", 1)[1].strip().split(maxsplit=1)[1],
+                    definition.split("}")[1].split(":", 1)[1].strip().split()[0]
+                )
+        except:
+            raise ValueError(
+                f'Malformed port definition, must be in the form `name : mode type`, got {definition}'
+            )
+
+class VHDLConstantsDirective(VHDLEntityIOGenericDirective):
+    id_title = 'genconstant'
+    title = 'Constants'
+    table_headers = 'Group', 'Constant', 'Type', 'Value', 'Description'
 
     def get_fields_from_definition(self, definition: str) -> Tuple[str, str, str]:
         try:
             return (
-                definition.split(":")[0].strip(),
-                definition.split(":", 1)[1].strip().split(maxsplit=1)[1],
-                definition.split(":", 1)[1].strip().split()[0]
+                definition.split(':')[0].strip(),
+                definition.split(':', 1)[1].split(':=')[0].strip(),
+                definition.split(':=')[1].strip()
             )
         except:
             raise ValueError(
-                f'Malformed port definition, must be in the form `name : mode type`, got {definition}'
+                f"Malformed constant definition, must be in the form `name : type := defaultValue`, got {definition}"
             )
 
 
 class VHDLParametersDirective(VHDLEntityIOGenericDirective):
     id_title = 'parameters'
     title = 'Parameters'
-    table_headers = 'Parameter', 'Type', 'Mode', 'Description'
+    table_headers = 'Group', 'Parameter', 'Type', 'Mode', 'Description'
 
     def get_fields_from_definition(self, definition: str) -> Tuple[str, str, str]:
         try:
@@ -253,17 +332,25 @@ class VHDLParametersDirective(VHDLEntityIOGenericDirective):
 
 
 class VHDLGenericsDirective(VHDLEntityIOGenericDirective):
-    id_title = 'genconstant'
+    id_title = 'gengeneric'
     title = 'Generics'
-    table_headers = 'Generic', 'Type', 'Default', 'Description'
+    table_headers = 'Group', 'Generic', 'Type', 'Default', 'Description'
 
-    def get_fields_from_definition(self, definition: str) -> Tuple[str, str, str]:
+    def get_fields_from_definition(self, definition: str) -> Union[Tuple[str, str, str], Tuple[str, str, str, str]]:
         try:
-            return (
-                definition.split(':')[0].strip(),
-                definition.split(':', 1)[1].split(':=')[0].strip(),
-                definition.split(':=')[1].strip()
+            if '}' not in definition:
+                return (
+                    definition.split(':')[0].strip(),
+                    definition.split(':', 1)[1].split(':=')[0].strip(),
+                    definition.split(':=')[1].strip()
             )
+            else:
+                return(
+                    definition.split("}")[0].strip(),
+                    definition.split("}")[1].split(":")[0].strip(),
+                    definition.split("}")[1].split(':', 1)[1].split(':=')[0].strip(),
+                    definition.split("}")[1].split(':=')[1].strip()
+                )
         except:
             raise ValueError(
                 f"Malformed generic definition, must be in the form `name : type := defaultValue`, got {definition}"
@@ -355,6 +442,16 @@ class VHDLAutoGenericsDirective(VHDLGenericsDirective):
         )
         return super().run()
 
+class VHDLAutoConstantsDirective(VHDLConstantsDirective):
+    has_content = False
+
+    def run(self):
+        init_autodoc(self.env.domains['vhdl'])
+        self.content = StringList(
+            [item for subitem in [[key, *[f'  {x}' for x in autodoc.constants[self.arguments[0].lower()][key]]]
+                                  for key in autodoc.constants[self.arguments[0].lower()].keys()] for item in subitem]
+        )
+        return super().run()
 
 class VHDLAutoTypeDirective(VHDLGeneralTypeDirective):
 
@@ -415,6 +512,7 @@ class VHDLDomain(Domain):
         'autoports': VHDLAutoPortsDirective,
         'autopackage': VHDLAutoPackageDirective,
         'autogenerics': VHDLAutoGenericsDirective,
+        'autoconstants': VHDLAutoConstantsDirective,
         'autorecord': VHDLAutoRecordDirective,
         'autoenum': VHDLAutoEnumDirective,
         'autotype': VHDLAutoTypeDirective,
@@ -423,6 +521,7 @@ class VHDLDomain(Domain):
         'parameters': VHDLParametersDirective,
         'ports': VHDLPortsDirective,
         'generics': VHDLGenericsDirective,
+        'constants': VHDLConstantsDirective,
         'package': VHDLPackagesDirective,
         'record': VHDLRecordTypeDirective,
         'recordelem': VHDLRecordElementDirective,
@@ -433,6 +532,7 @@ class VHDLDomain(Domain):
         'refs': {
             'types': defaultdict(list),
             'portsignal': defaultdict(list),
+            'gengeneric': defaultdict(list),
             'genconstant': defaultdict(list),
             'parameters': defaultdict(list),
             'entity': defaultdict(list),
@@ -445,6 +545,7 @@ class VHDLDomain(Domain):
     roles = {
         'portsignal': XRefRole(),
         'genconstant': XRefRole(),
+        'gengeneric': XRefRole(),
         'type': XRefRole(),
         'entity': XRefRole(),
     }
@@ -455,6 +556,8 @@ class VHDLDomain(Domain):
             index = self.data['refs']['types']
         elif typ == 'portsignal':
             index = self.data['refs']['portsignal']
+        elif typ == 'gengeneric':
+            index = self.data['refs']['gengeneric']
         elif typ == 'genconstant':
             index = self.data['refs']['genconstant']
         elif typ == 'entity':
